var LiveClick =
{
	// Constants
	PREF_BASE : "extensions.liveclick.",

	// Class properties
	_strings : null,

	// Initialization code
	onLoad : function ()
	{
		// Load localized strings
		this._strings = document.getElementById( "liveclick-strings" );
	},

	// Extends the onClick behavior of a bookmark
	onClick : function (aEvent, aDS)
	{
		// Fix 0.1.5.2: Break out if clicking on a non-bookmark item
		// i.e. menuseparator, "Bookmark This Page," "Manage Bookmarks", etc.
		if (!aEvent.target.id) return;

		var iButton = aEvent.button;
		var bAccel = aEvent.ctrlKey;
		var sLocal = aEvent.target.localName;

		var iOnLeft = this.getPreference( "onLeftClick", "int", 1 );
		var iOnMiddle = this.getPreference( "onMiddleClick", "int", 2 );

		var iAction = -1;
		// -1 => Do browser default
		//  0 => Do nothing
		//  1 => Open location in current window/tab
		//  2 => Open location in new tab
		//  3 => View feed in current window/tab
		//  4 => View feed in new tab
		//  5 => Open live bookmark(s) in tab(s)

		if (bAccel)
			// Treat accel click like middle click
			iAction = iOnMiddle;
		else if (iButton == 0 && iOnLeft > 0 && sLocal != "toolbarbutton")
			// Set left click action if
			//	1) action NOT browser default and NOT do nothing (default browser action)
			//	2) bookmark NOT located directly on bookmarks toolbar
			iAction = iOnLeft;
		else if (iButton == 1 && iOnMiddle != -1 && iOnMiddle != 5)
			// Set middle click action if
			//	1) action NOT browser default
			//	2) NOT "Open in Tabs" (default browser action)
			iAction = iOnMiddle;

		// Continue if action NOT browser default
		if (iAction >= 0)
		{
			// Get container type
			var rResource = RDF.GetResource( aEvent.target.id );
			var sType = BookmarksUtils.resolveType( rResource );

			// Process only on livemarks
			if (sType == "Livemark")
			{
				if (iAction == 0) return; // Do nothing

				var sTarget = "";
				// Even actions open bookmark in new tab
				// Odd actions open bookmark in current tab/window
				iAction % 2 == 0 ? sTarget = "tab" : sTarget = "current";

				if (iAction == 1 || iAction == 2)
					// Open location
					this.loadBookmark( aEvent, aDS, "URL", sTarget );
				else if (iAction == 3 || iAction == 4)
					// View feed
					this.loadBookmark( aEvent, aDS, "FeedURL", sTarget );
				else if (iAction == 5)
					// Open in tabs
					this.loadGroupBookmark( aEvent );

				// Bug forces us to close menus manually
				if (!bAccel) this.closeMenus( aEvent.target );
				return;
			}
		}

		// If you reach this point, perform default middle click action(s)
		BookmarksMenu.loadBookmarkMiddleClick( aEvent, aDS )
	},

	// Extends the BookmarksMenu.onShowMenu method to show LiveClick menu items
	onShowMenu : function (aEvent, aDS)
	{
		var aTarget = aEvent.target;
		BookmarksMenu.showOpenInTabsMenuItem( aTarget );

		// Get container type
		var rResource = RDF.GetResource( aTarget.parentNode.id )
		var sType = BookmarksUtils.resolveType( rResource );

		// Continue only if container is a livemark
		if (sType == "Livemark")
		{
			var bShowPopup = this.getPreference( "showPopupAny", "bool", true );
			var bShowOpen = this.getPreference( "showPopupOpen", "bool", true )
								&& this.hasLocation( aTarget.parentNode.id );
			var bShowFeed = this.getPreference( "showPopupFeed", "bool", true );

			// Continue only if LiveClick menu items are enabled in options
			if (bShowPopup && (bShowOpen || bShowFeed))
			{
				// Continue only if menu items are done loading
				// ie. avoid separating LiveClick menu elements during refreshLivemark
				if (!this.isLoading( aTarget ))
				{
					var nodeList = aTarget.getElementsByAttribute( "class", "liveclick-menuitem" );

					// Add menu items if menu NOT already altered by LiveClick
					if (nodeList.length == 0)
						this.addMenuItems( aTarget, bShowOpen, bShowFeed );
				}
			}
		}
		// Show favicons if a regular bookmarks menu (added 0.1.5)
		else
		{
			var bShowFavIcons = this.getPreference( "showFavIcons", "bool", true );

			// Continue only if Show FavIcons enabled in options
			if (bShowFavIcons) this.showFavIcons( aTarget.parentNode, aDS );
		}

		// Fix 0.1.5.1: Removed aDS argument from showEmptyItem
		BookmarksMenu.showEmptyItem( aTarget );
	},

	// Extends the BookmarksMenu.onHideMenu method to hide LiveClick menu items
	onHideMenu : function (aEvent)
	{
		var aTarget = aEvent.target;
		BookmarksMenu.hideOpenInTabsMenuItem( aTarget );

		if (aTarget.hasChildNodes())
		{
			var child = aTarget.lastChild;
			var removed = 0;
			while (child)
			{
				var sClass = child.getAttribute( "class" );
				if (sClass == "liveclick-menuitem" || sClass == "liveclick-menuseparator")
				{
					var prevchild = child.previousSibling;
					aTarget.removeChild( child );
					child = prevchild;
					removed++;
					if (removed == 3) break;	// Max number of LiveClick menu elements
				}
				else
				{
					child = child.previousSibling;
				}
			}
		}

		BookmarksMenu.hideEmptyItem( aTarget );
	},

	// Open URL
	openURL : function (sURL, aEvent, sTarget)
	{
		if (sURL == "") return;

		if (sTarget != '')
			openUILinkIn( sURL, sTarget );
		else
			openUILink( sURL, aEvent );

		if (content) content.focus();
		aEvent.preventBubble();
	},

	loadBookmark : function (aEvent, aDS, sField, sTarget)
	{
		var aTarget = aEvent.target;
		if (aTarget.getAttribute( "class" ) == "liveclick-menuitem")
			aTarget = aTarget.parentNode.parentNode;

		var rResource = RDF.GetResource( aTarget.id );
		var aSelection = BookmarksUtils.getSelectionFromResource( rResource );

		// Executed only on the first selection
		var uri = aSelection.item[0].Value;
		var sURL = BookmarksUtils.getProperty( uri, gNC_NS + sField, aDS );

		// Use FeedURL if sURL is empty (added 0.1.5)
		// Suggested by SilentJ
		//   http://forums.mozillazine.org/viewtopic.php?t=416543#2278294
		if (sURL == "") sURL = BookmarksUtils.getProperty( uri, gNC_NS + "FeedURL", aDS );

		this.openURL( sURL, aEvent, sTarget );
	},

	loadBookmarkClick : function (aEvent, aDS, sField)
	{
		// Perform on middle click only
		if (aEvent.button != 1) return;

		this.loadBookmark( aEvent, aDS, sField, '' );
		this.closeMenus( aEvent.target );	// Bug forces us to close menus manually
	},

	loadGroupBookmark : function (aEvent)
	{
		var aTarget = aEvent.target;
		var rResource = RDF.GetResource( aTarget.id );
		var aSelection = BookmarksUtils.getSelectionFromResource( rResource );

		// Executed only on the first selection
		var uri = aSelection.item[0].Value;

		BookmarksCommand.openGroupBookmark( uri, "current" );
	},

	// Returns true if bookmark has a location (URL)
	hasLocation : function (aId)
	{
		var uri = RDF.GetResource( aId );
		var sURL = BookmarksUtils.getProperty( uri, gNC_NS + "URL" );

		return sURL != "";
	},

	// Returns true if popup menu is still loading
	isLoading : function (aTarget)
	{
		var bLoading = true;

		// Test if valid for tabs
		// BUG: Won't display LiveClick menu elements if livemark holds only 1 static bookmark
		// 		a consequence of using validOpenInTabsMenuItem (see: 257285)
		var bLoading = !BookmarksMenu.validOpenInTabsMenuItem( aTarget );

		if (!bLoading) return false;

		// Test if livemark has failed to load
		var nodeList =
			aTarget.getElementsByAttribute( "label", BookmarksUtils.getLocaleString( "BookmarksLivemarkFailed" ) );
		bLoading = nodeList.length == 0;

		return bLoading;
	},

	// Add LiveClick menu elements to popup (aTarget)
	addMenuItems : function (aTarget, bShowOpen, bShowFeed)
	{
		// Create LiveClick menu elements
		var nSep1 = document.createElementNS( gXUL_NS, "menuseparator" );
		nSep1.setAttribute( "class", "liveclick-menuseparator" );

		var nOpen = document.createElementNS( gXUL_NS, "menuitem" );
		nOpen.setAttribute( "class", "liveclick-menuitem" );
		nOpen.setAttribute( "label", this._strings.getString( "open" ) );
		nOpen.setAttribute( "accesskey", this._strings.getString( "openKey" ) );
		nOpen.setAttribute( "oncommand", "LiveClick.loadBookmark(event, this.database, 'URL', '');" );
		nOpen.setAttribute( "onclick", "LiveClick.loadBookmarkClick(event, this.database, 'URL');" );

		var nFeed = document.createElementNS( gXUL_NS, "menuitem" );
		nFeed.setAttribute( "class", "liveclick-menuitem" );
		nFeed.setAttribute( "label", this._strings.getString( "feed" ) );
		nFeed.setAttribute( "accesskey", this._strings.getString( "feedKey" ) );
		nFeed.setAttribute( "oncommand", "LiveClick.loadBookmark(event, this.database, 'FeedURL', '');" );
		nFeed.setAttribute( "onclick", "LiveClick.loadBookmarkClick(event, this.database, 'FeedURL');" );

		// Get placement preference
		var bShowAtTop = this.getPreference( "showAtTop", "int", 1 ) == 1;

		// Test for empty menu
		var bHasChildren = aTarget.hasChildNodes();

		if (bShowAtTop && bHasChildren)
		{
			// Insert enabled elements at top of menu
			var nFirst = aTarget.firstChild;
			if (bShowOpen) aTarget.insertBefore( nOpen, nFirst );
			if (bShowFeed) aTarget.insertBefore( nFeed, nFirst );
			aTarget.insertBefore( nSep1, nFirst );
		}
		else
		{
			// Add enabled elements to menu
			if (bHasChildren) aTarget.appendChild( nSep1 );
			if (bShowOpen) aTarget.appendChild( nOpen );
			if (bShowFeed) aTarget.appendChild( nFeed );
		}
	},

	// Show favicons instead of RSS icons for all livemarks (added 0.1.5)
	showFavIcons : function (aTarget, aDS)
	{
		// Get all livemarks in menu
		var nodeList =
			aTarget.getElementsByAttribute( "type", "http://home.netscape.com/NC-rdf#Livemark" );

		// Find livemarks with favicons
		for (var i = 0; i < nodeList.length; i++)
		{
			var node = nodeList[i];

			var uri = RDF.GetResource( node.id );
			var icon = BookmarksUtils.getProperty( uri, gNC_NS + "Icon", aDS );

			// If favicon available, use in place of RSS icon
			if (icon)
			{
				// Need menu-iconic class, otherwise no icons at all
				// Add liveclick-iconic class so icons show at 16x16 pixels
				node.setAttribute( "class", "bookmark-item menu-iconic liveclick-iconic" );
				node.setAttribute( "image", icon );
			}
		}
	},

	// Build feed button context menu (added 0.1.5)
	previewFeedMenu : function (aTarget)
	{
		// Retrive feeds info
		var feeds = gBrowser.mCurrentBrowser.feeds;

		// No feeds, no menu
		if (feeds.length == 0) return false;

		// Reset menu
		while (aTarget.hasChildNodes())
			aTarget.removeChild( aTarget.lastChild );

		// Build the menu showing the available feed choices for preview
		for (var i = 0; i < feeds.length; i++)
		{
			var feedInfo = feeds[i];
			var menuItem = document.createElement( "menuitem" );
			var baseTitle = feedInfo.title || feedInfo.href;
			var labelStr = this._strings.getFormattedString( "previewFeed", [baseTitle] );
			menuItem.setAttribute( "label", labelStr );
			menuItem.setAttribute( "feed", feedInfo.href );
			menuItem.setAttribute( "tooltiptext", feedInfo.href );
			aTarget.appendChild( menuItem );
		}

		return true;
	},

	// Load feed (added 0.1.5)
	previewFeed : function (aEvent, aTarget)
	{
  		var sURL = aEvent.target.getAttribute( "feed" );
		this.openURL( sURL, aEvent, '' );

		// If middle click, close popup manually
		if (aEvent.button == 1) aTarget.hidePopup();
	},

	// Clicking a bookmarks menu item leaves top level menu active
	// Based on fix from https://bugzilla.mozilla.org/show_bug.cgi?id=307796
	closeMenus : function (node)
	{
		BookmarksMenuDNDObserver.mCurrentDragOverTarget = null;
		BookmarksMenuDNDObserver.onDragCloseTarget();

		while ("tagName" in node)
		{
			if (node.tagName == "menupopup" || node.tagName == "popup")
				node.hidePopup();

			node.removeAttribute( '_moz-menuactive' );
			node = node.parentNode;
		}
	},

	getPreference : function (sString, sType, sDefault)
	{
		var service = Components.classes["@mozilla.org/preferences-service;1"].
						getService( Components.interfaces.nsIPrefService ).
						getBranch( "" );

		var sName = this.PREF_BASE + sString;
		var sValue = null;

		try
		{
			switch (sType)
			{
				case "bool":
					sValue = service.getBoolPref( sName );
					break;
				case "int":
					sValue = service.getIntPref( sName );
					break;
				default:
					sValue = service.getCharPref( sName );
					break;
			}
		}
		catch (e)
		{
			// Do Nothing
		}

		if (sValue == null) sValue = sDefault;

		return sValue;
	}
}

// Register LiveClick as an onLoad listener. To avoid getting called
// multiple times, register in non-capturing mode (3rd param = false).
window.addEventListener( "load", function() { LiveClick.onLoad(); }, false );